@@ -129,7 +129,7 @@ void ChangeOwnerTrustCommand::doStart()
     }
 
     if (key.ownerTrust() < Key::OwnerTrust::Full) {
-        const auto text = (DeVSCompliance::isCompliant() && Formatting::isKeyDeVs(key))
+        const auto text = (DeVSCompliance::isCompliant() && DeVSCompliance::allSubkeysAreCompliant(key))
             ? xi18nc("@info %1: a certificate, %2: name of a compliance mode",
                      "<para>Do you want to grant '%1' the power to mark certificates as %2 for you?</para>"
                      "<para><emphasis>This means that the owner of this certificate properly checks fingerprints "
@@ -152,7 +152,7 @@ void ChangeOwnerTrustCommand::doStart()
             d->canceled();
         }
     } else {
-        const auto text = (DeVSCompliance::isCompliant() && Formatting::isKeyDeVs(key))
+        const auto text = (DeVSCompliance::isCompliant() && DeVSCompliance::allSubkeysAreCompliant(key))
             ? xi18nc("@info %1: a certificate, %2: name of a compliance mode",
                      "<para>The certificate '%1' is empowered to mark other certificates as %2 for you.</para>"
                      "<para>Do you want to revoke this power?</para>",
@@ -238,21 +238,21 @@ void ChangeOwnerTrustCommand::Private::showErrorDialog(const Error &err)
                      "<para>An error occurred while marking certificate '%1' as your certificate.</para>"
                      "<para><message>%2</message></para>",
                      keyInfo,
-                     QString::fromUtf8(err.asString())));
+                     Formatting::errorAsString(err)));
         break;
     case Key::OwnerTrust::Full:
         error(xi18nc("@info",
                      "<para>An error occurred while granting certification power to '%1'.</para>"
                      "<para><message>%2</message></para>",
                      keyInfo,
-                     QString::fromUtf8(err.asString())));
+                     Formatting::errorAsString(err)));
         break;
     default:
         error(xi18nc("@info",
                      "<para>An error occurred while revoking the certification power of '%1'.</para>"
                      "<para><message>%2</message></para>",
                      keyInfo,
-                     QString::fromUtf8(err.asString())));
+                     Formatting::errorAsString(err)));
     }
 }
 
