@@ -201,7 +201,7 @@ void ExportSecretKeyCommand::Private::cancel()
 
 std::unique_ptr<QGpgME::ExportJob> ExportSecretKeyCommand::Private::startExportJob(const Key &key)
 {
-#ifdef QGPGME_SUPPORTS_SECRET_KEY_EXPORT
+#if QGPGME_SUPPORTS_SECRET_KEY_EXPORT
     const bool armor = key.protocol() == GpgME::OpenPGP && filename.endsWith(u".asc", Qt::CaseInsensitive);
     const QGpgME::Protocol *const backend = (key.protocol() == GpgME::OpenPGP) ? QGpgME::openpgp() : QGpgME::smime();
     Q_ASSERT(backend);
@@ -216,8 +216,13 @@ std::unique_ptr<QGpgME::ExportJob> ExportSecretKeyCommand::Private::startExportJ
             q, [this](const GpgME::Error &err, const QByteArray &keyData) {
                 onExportJobResult(err, keyData);
             });
-    connect(exportJob.get(), &QGpgME::Job::progress,
+#if QGPGME_JOB_HAS_NEW_PROGRESS_SIGNALS
+    connect(exportJob.get(), &QGpgME::Job::jobProgress,
             q, &Command::progress);
+#else
+    connect(exportJob.get(), &QGpgME::Job::progress,
+            q, [this](const QString &, int current, int total) { Q_EMIT q->progress(current, total); });
+#endif
 
     const GpgME::Error err = exportJob->start({QLatin1String{key.primaryFingerprint()}});
     if (err) {
@@ -279,7 +284,7 @@ void ExportSecretKeyCommand::Private::showError(const Error &err)
     error(xi18nc("@info",
                  "<para>An error occurred during the backup of the secret key:</para>"
                  "<para><message>%1</message></para>",
-                 QString::fromLocal8Bit(err.asString())),
+                 Formatting::errorAsString(err)),
           errorCaption());
 }
 
