@@ -62,6 +62,9 @@ public:
     /** Check if the text is empty */
     bool isEmpty() const;
 
+    /** Returns true, if the user is editing the input. */
+    bool isEditingInProgress() const;
+
     /** Returns true if the field is empty or if a key or group is selected. */
     bool hasAcceptableInput() const;
 
@@ -83,6 +86,9 @@ Q_SIGNALS:
     /** Emitted when the entry is no longer empty. */
     void editingStarted();
 
+    /** Emitted when the input is cleared (i.e. becomes empty). */
+    void cleared();
+
     /** Emitted when the certificate selection dialog is requested. */
     void certificateSelectionRequested();
 
