@@ -230,7 +230,7 @@ static QString makeResultDetails(const SigningResult &result, const QString &inp
     }
 
     if (err || err.isCanceled()) {
-        return QString::fromLocal8Bit(err.asString()).toHtmlEscaped();
+        return Formatting::errorAsString(err).toHtmlEscaped();
     }
     return QString();
 }
@@ -247,7 +247,7 @@ static QString makeResultDetails(const EncryptionResult &result, const QString &
     }
 
     if (err || err.isCanceled()) {
-        return QString::fromLocal8Bit(err.asString()).toHtmlEscaped();
+        return Formatting::errorAsString(err).toHtmlEscaped();
     }
     return i18n(" Encryption succeeded.");
 }
@@ -280,6 +280,11 @@ public:
     explicit Private(SignEncryptTask *qq);
 
 private:
+    QString inputLabel() const;
+    QString outputLabel() const;
+
+    bool removeExistingOutputFile();
+
     void startSignEncryptJob(GpgME::Protocol proto);
     std::unique_ptr<QGpgME::SignJob> createSignJob(GpgME::Protocol proto);
     std::unique_ptr<QGpgME::SignEncryptJob> createSignEncryptJob(GpgME::Protocol proto);
@@ -317,6 +322,7 @@ private:
     bool archive  : 1;
 
     QPointer<QGpgME::Job> job;
+    QString labelText;
     std::shared_ptr<OverwritePolicy> m_overwritePolicy;
 };
 
@@ -327,14 +333,14 @@ SignEncryptTask::Private::Private(SignEncryptTask *qq)
     , detached{false}
     , clearsign{false}
     , archive{false}
-    , m_overwritePolicy{new OverwritePolicy{nullptr}}
+    , m_overwritePolicy{new OverwritePolicy{OverwritePolicy::Ask}}
 {
     q->setAsciiArmor(true);
 }
 
 std::shared_ptr<const Task::Result> SignEncryptTask::Private::makeErrorResult(const Error &err, const QString &errStr, const AuditLogEntry &auditLog)
 {
-    return std::shared_ptr<const ErrorResult>(new ErrorResult(sign, encrypt, err, errStr, q->label(), output ? output->label() : QString{}, auditLog));
+    return std::shared_ptr<const ErrorResult>(new ErrorResult(sign, encrypt, err, errStr, inputLabel(), outputLabel(), auditLog));
 }
 
 SignEncryptTask::SignEncryptTask(QObject *p)
@@ -380,6 +386,11 @@ void SignEncryptTask::setOutputFileName(const QString &fileName)
     d->outputFileName = fileName;
 }
 
+QString SignEncryptTask::outputFileName() const
+{
+    return d->outputFileName;
+}
+
 void SignEncryptTask::setSigners(const std::vector<Key> &signers)
 {
     kleo_assert(!d->job);
@@ -452,13 +463,10 @@ Protocol SignEncryptTask::protocol() const
 
 QString SignEncryptTask::label() const
 {
-    if (d->input) {
-        return d->input->label();
-    } else if (!d->inputFileNames.empty()) {
-        const auto firstFile = QFileInfo{d->inputFileNames.front()}.fileName();
-        return d->inputFileNames.size() == 1 ? firstFile : i18nc("<name of first file>, ...", "%1, ...", firstFile);
+    if (!d->labelText.isEmpty()) {
+        return d->labelText;
     }
-    return {};
+    return d->inputLabel();
 }
 
 QString SignEncryptTask::tag() const
@@ -488,12 +496,6 @@ void SignEncryptTask::doStart()
         }
     }
 
-    if (!d->output) {
-        d->output = Output::createFromFile(d->outputFileName, d->m_overwritePolicy);
-    }
-    // release grab on policy, so that output can infer multiple outputs from use count > 1
-    d->m_overwritePolicy.reset();
-
     const auto proto = protocol();
 #if QGPGME_SUPPORTS_ARCHIVE_JOBS
     if (d->archive && archiveJobsCanBeUsed(proto)) {
@@ -501,10 +503,55 @@ void SignEncryptTask::doStart()
     } else
 #endif
     {
+        if (!d->output) {
+            d->output = Output::createFromFile(d->outputFileName, d->m_overwritePolicy);
+        }
         d->startSignEncryptJob(proto);
     }
 }
 
+QString SignEncryptTask::Private::inputLabel() const
+{
+    if (input) {
+        return input->label();
+    }
+    if (!inputFileNames.empty()) {
+        const auto firstFile = QFileInfo{inputFileNames.front()}.fileName();
+        return inputFileNames.size() == 1 ? firstFile : i18nc("<name of first file>, ...", "%1, ...", firstFile);
+    }
+    return {};
+}
+
+QString SignEncryptTask::Private::outputLabel() const
+{
+    return output ? output->label() : QFileInfo{outputFileName}.fileName();
+}
+
+bool SignEncryptTask::Private::removeExistingOutputFile()
+{
+    if (QFile::exists(outputFileName)) {
+        bool fileRemoved = false;
+        // we should already have asked the user for overwrite permission
+        if (m_overwritePolicy && (m_overwritePolicy->policy() == OverwritePolicy::Overwrite)) {
+            qCDebug(KLEOPATRA_LOG) << __func__ << "going to remove file for overwriting" << outputFileName;
+            fileRemoved = QFile::remove(outputFileName);
+            if (!fileRemoved) {
+                qCDebug(KLEOPATRA_LOG) << __func__ << "removing file to overwrite failed";
+            }
+        } else {
+            qCDebug(KLEOPATRA_LOG) << __func__ << "we have no permission to overwrite" << outputFileName;
+        }
+        if (!fileRemoved) {
+            QMetaObject::invokeMethod(q, [this]() {
+                slotResult(nullptr, SigningResult{}, EncryptionResult{Error::fromCode(GPG_ERR_EEXIST)});
+            }, Qt::QueuedConnection);
+            return false;
+        }
+    }
+
+    return true;
+}
+
 void SignEncryptTask::Private::startSignEncryptJob(GpgME::Protocol proto)
 {
     kleo_assert(input);
@@ -559,6 +606,7 @@ void SignEncryptTask::Private::startSignEncryptJob(GpgME::Protocol proto)
 
 void SignEncryptTask::cancel()
 {
+    qCDebug(KLEOPATRA_LOG) << this << __func__;
     if (d->job) {
         d->job->slotCancel();
     }
@@ -622,6 +670,11 @@ std::unique_ptr<QGpgME::EncryptJob> SignEncryptTask::Private::createEncryptJob(G
 void SignEncryptTask::Private::startSignEncryptArchiveJob(GpgME::Protocol proto)
 {
     kleo_assert(!input);
+    kleo_assert(!output);
+
+#if ! QGPGME_ARCHIVE_JOBS_SUPPORT_OUTPUT_FILENAME
+    output = Output::createFromFile(outputFileName, m_overwritePolicy);
+#endif
 
     const auto baseDirectory = heuristicBaseDirectory(inputFileNames);
     if (baseDirectory.isEmpty()) {
@@ -639,28 +692,61 @@ void SignEncryptTask::Private::startSignEncryptArchiveJob(GpgME::Protocol proto)
             qCDebug(KLEOPATRA_LOG) << "Adding symmetric flag";
         }
         if (sign) {
+            labelText = i18nc("@info", "Creating signed and encrypted archive ...");
             std::unique_ptr<QGpgME::SignEncryptArchiveJob> job = createSignEncryptArchiveJob(proto);
             kleo_assert(job.get());
             job->setBaseDirectory(baseDirectory);
-
+#if QGPGME_ARCHIVE_JOBS_SUPPORT_OUTPUT_FILENAME
+            job->setSigners(signers);
+            job->setRecipients(recipients);
+            job->setInputPaths(relativePaths);
+            job->setOutputFile(outputFileName);
+            job->setEncryptionFlags(flags);
+            if (!removeExistingOutputFile()) {
+                return;
+            }
+            job->startIt();
+#else
             job->start(signers, recipients, relativePaths, output->ioDevice(), flags);
+#endif
 
             this->job = job.release();
         } else {
+            labelText = i18nc("@info", "Creating encrypted archive ...");
             std::unique_ptr<QGpgME::EncryptArchiveJob> job = createEncryptArchiveJob(proto);
             kleo_assert(job.get());
             job->setBaseDirectory(baseDirectory);
-
+#if QGPGME_ARCHIVE_JOBS_SUPPORT_OUTPUT_FILENAME
+            job->setRecipients(recipients);
+            job->setInputPaths(relativePaths);
+            job->setOutputFile(outputFileName);
+            job->setEncryptionFlags(flags);
+            if (!removeExistingOutputFile()) {
+                return;
+            }
+            job->startIt();
+#else
             job->start(recipients, relativePaths, output->ioDevice(), flags);
+#endif
 
             this->job = job.release();
         }
     } else if (sign) {
+        labelText = i18nc("@info", "Creating signed archive ...");
         std::unique_ptr<QGpgME::SignArchiveJob> job = createSignArchiveJob(proto);
         kleo_assert(job.get());
         job->setBaseDirectory(baseDirectory);
-
+#if QGPGME_ARCHIVE_JOBS_SUPPORT_OUTPUT_FILENAME
+        job->setSigners(signers);
+        job->setInputPaths(relativePaths);
+        job->setOutputFile(outputFileName);
+        if (!removeExistingOutputFile()) {
+            return;
+        }
+        job->startIt();
+#else
         job->start(signers, relativePaths, output->ioDevice());
+#endif
 
         this->job = job.release();
     } else {
@@ -731,17 +817,34 @@ void SignEncryptTask::Private::slotResult(const QGpgME::Job *job, const SigningR
     const AuditLogEntry auditLog = AuditLogEntry::fromJob(job);
     bool outputCreated = false;
     if (input && input->failed()) {
-        output->cancel();
+        if (output) {
+            output->cancel();
+        }
         q->emitResult(makeErrorResult(Error::fromCode(GPG_ERR_EIO),
                                       i18n("Input error: %1", escape( input->errorString())),
                                       auditLog));
         return;
     } else if (sresult.error().code() || eresult.error().code()) {
-        output->cancel();
+        if (output) {
+            output->cancel();
+        }
+        if (!outputFileName.isEmpty() && eresult.error().code() != GPG_ERR_EEXIST) {
+            // ensure that the output file is removed if the task was canceled or an error occurred;
+            // unless a "file exists" error occurred because this means that the file with the name
+            // of outputFileName wasn't created as result of this task
+            if (QFile::exists(outputFileName)) {
+                qCDebug(KLEOPATRA_LOG) << __func__ << "Removing output file" << outputFileName << "after error or cancel";
+                if (!QFile::remove(outputFileName)) {
+                    qCDebug(KLEOPATRA_LOG) << __func__ << "Removing output file" << outputFileName << "failed";
+                }
+            }
+        }
     } else {
         try {
             kleo_assert(!sresult.isNull() || !eresult.isNull());
-            output->finalize();
+            if (output) {
+                output->finalize();
+            }
             outputCreated = true;
             if (input) {
                 input->finalize();
@@ -752,8 +855,8 @@ void SignEncryptTask::Private::slotResult(const QGpgME::Job *job, const SigningR
         }
     }
 
-    const LabelAndError inputInfo{q->label(), input ? input->errorString() : QString{}};
-    const LabelAndError outputInfo{output->label(), output->errorString()};
+    const LabelAndError inputInfo{inputLabel(), input ? input->errorString() : QString{}};
+    const LabelAndError outputInfo{outputLabel(), output ? output->errorString() : QString{}};
     q->emitResult(std::shared_ptr<Result>(new SignEncryptFilesResult(sresult, eresult, inputInfo, outputInfo, outputCreated, auditLog)));
 }
 
