@@ -9,6 +9,7 @@
 
 #include <config-kleopatra.h>
 
+#include "searchbar.h"
 #include "tabwidget.h"
 #include "keytreeview.h"
 #include "kleopatra_debug.h"
@@ -473,7 +474,7 @@ TabWidget::Private::Private(TabWidget *qq)
     // create "Close Tab" button after tab widget to ensure correct tab order
     closeTabButton = new QToolButton{q};
 
-    connect(tabWidget, SIGNAL(currentChanged(int)), q, SLOT(currentIndexChanged(int)));
+    connect(tabWidget, &QTabWidget::currentChanged, q, [this](int index) { currentIndexChanged(index); });
     connect(tabWidget->tabBar(), &QWidget::customContextMenuRequested, q, [this](const QPoint & p) {
         slotContextMenu(p);
     });
@@ -691,7 +692,6 @@ void TabWidget::Private::collapseAll(Page *page)
 TabWidget::TabWidget(QWidget *p, Qt::WindowFlags f)
     : QWidget(p, f), d(new Private(this))
 {
-
 }
 
 TabWidget::~TabWidget() {
@@ -811,7 +811,7 @@ void TabWidget::createActions(KActionCollection *coll)
     }
     const action_data actionDataNew = {
         "window_new_tab", i18n("New Tab"), i18n("Open a new tab"),
-        "tab-new-background", this, SLOT(slotNewTab()), QStringLiteral("CTRL+SHIFT+N"), false, true
+        "tab-new-background", this, [this](bool) { d->slotNewTab(); }, QStringLiteral("CTRL+SHIFT+N")
     };
 
     d->newAction = make_action_from_data(actionDataNew, coll);
@@ -819,35 +819,35 @@ void TabWidget::createActions(KActionCollection *coll)
     const std::vector<action_data> actionData = {
         {
             Actions::Rename, i18n("Rename Tab..."), i18n("Rename this tab"),
-            "edit-rename", this, SLOT(slotRenameCurrentTab()), QStringLiteral("CTRL+SHIFT+R"), false, false
+            "edit-rename", this, [this](bool) { d->slotRenameCurrentTab(); }, QStringLiteral("CTRL+SHIFT+R"), RegularQAction, Disabled
         },
         {
             Actions::Duplicate, i18n("Duplicate Tab"), i18n("Duplicate this tab"),
-            "tab-duplicate", this, SLOT(slotDuplicateCurrentTab()), QStringLiteral("CTRL+SHIFT+D"), false, true
+            "tab-duplicate", this, [this](bool) { d->slotDuplicateCurrentTab(); }, QStringLiteral("CTRL+SHIFT+D")
         },
         {
             Actions::Close, i18n("Close Tab"), i18n("Close this tab"),
-            "tab-close", this, SLOT(slotCloseCurrentTab()), QStringLiteral("CTRL+SHIFT+W"), false, false
+            "tab-close", this, [this](bool) { d->slotCloseCurrentTab(); }, QStringLiteral("CTRL+SHIFT+W"), RegularQAction, Disabled
         }, // ### CTRL-W when available
         {
             Actions::MoveLeft, i18n("Move Tab Left"), i18n("Move this tab left"),
-            nullptr, this, SLOT(slotMoveCurrentTabLeft()), QStringLiteral("CTRL+SHIFT+LEFT"), false, false
+            nullptr, this, [this](bool) { d->slotMoveCurrentTabLeft(); }, QStringLiteral("CTRL+SHIFT+LEFT"), RegularQAction, Disabled
         },
         {
             Actions::MoveRight, i18n("Move Tab Right"), i18n("Move this tab right"),
-            nullptr, this, SLOT(slotMoveCurrentTabRight()), QStringLiteral("CTRL+SHIFT+RIGHT"), false, false
+            nullptr, this, [this](bool) { d->slotMoveCurrentTabRight(); }, QStringLiteral("CTRL+SHIFT+RIGHT"), RegularQAction, Disabled
         },
         {
             Actions::Hierarchical, i18n("Hierarchical Certificate List"), QString(),
-            nullptr, this, SLOT(slotToggleHierarchicalView(bool)), QString(), true, false
+            nullptr, this, [this](bool on) { d->slotToggleHierarchicalView(on); }, QString(), KFToggleAction, Disabled
         },
         {
             Actions::ExpandAll, i18n("Expand All"), QString(),
-            nullptr, this, SLOT(slotExpandAll()), QStringLiteral("CTRL+."), false, false
+            nullptr, this, [this](bool) { d->slotExpandAll(); }, QStringLiteral("CTRL+."), RegularQAction, Disabled
         },
         {
             Actions::CollapseAll, i18n("Collapse All"), QString(),
-            nullptr, this, SLOT(slotCollapseAll()), QStringLiteral("CTRL+,"), false, false
+            nullptr, this, [this](bool) { d->slotCollapseAll(); }, QStringLiteral("CTRL+,"), RegularQAction, Disabled
         },
     };
 
@@ -862,7 +862,7 @@ void TabWidget::createActions(KActionCollection *coll)
         if (ad.icon) {
             action->setIcon(QIcon::fromTheme(QLatin1String(ad.icon)));
         }
-        action->setEnabled(ad.enabled);
+        action->setEnabled(ad.actionState == Enabled);
         d->otherPageActions.insert(ad.name, action);
     }
 
@@ -913,10 +913,10 @@ QTreeView *TabWidget::Private::addView(Page *page, Page *columnReference)
     page->setFlatModel(flatModel);
     page->setHierarchicalModel(hierarchicalModel);
 
-    connect(page, SIGNAL(titleChanged(QString)), q, SLOT(slotPageTitleChanged(QString)));
-    connect(page, SIGNAL(keyFilterChanged(std::shared_ptr<Kleo::KeyFilter>)), q, SLOT(slotPageKeyFilterChanged(std::shared_ptr<Kleo::KeyFilter>)));
-    connect(page, SIGNAL(stringFilterChanged(QString)), q, SLOT(slotPageStringFilterChanged(QString)));
-    connect(page, SIGNAL(hierarchicalChanged(bool)), q, SLOT(slotPageHierarchyChanged(bool)));
+    connect(page, &Page::titleChanged, q, [this](const QString &text) { slotPageTitleChanged(text); });
+    connect(page, &Page::keyFilterChanged, q, [this](const std::shared_ptr<Kleo::KeyFilter> &filter) { slotPageKeyFilterChanged(filter); });
+    connect(page, &Page::stringFilterChanged, q, [this](const QString &text) { slotPageStringFilterChanged(text); });
+    connect(page, &Page::hierarchicalChanged, q, [this](bool on) { slotPageHierarchyChanged(on); });
 
     if (columnReference) {
         page->setColumnSizes(columnReference->columnSizes());
@@ -988,22 +988,16 @@ void TabWidget::saveViews(KConfig *config) const
     }
 }
 
-static void xconnect(const QObject *o1, const char *signal, const QObject *o2, const char *slot)
+void TabWidget::connectSearchBar(SearchBar *sb)
 {
-    QObject::connect(o1, signal, o2, slot);
-    QObject::connect(o2, signal, o1, slot);
-}
+    connect(sb, &SearchBar::stringFilterChanged, this, &TabWidget::setStringFilter);
+    connect(this, &TabWidget::stringFilterChanged, sb, &SearchBar::setStringFilter);
 
-void TabWidget::connectSearchBar(QObject *sb)
-{
-    xconnect(sb, SIGNAL(stringFilterChanged(QString)),
-             this, SLOT(setStringFilter(QString)));
-    xconnect(sb, SIGNAL(keyFilterChanged(std::shared_ptr<Kleo::KeyFilter>)),
-             this, SLOT(setKeyFilter(std::shared_ptr<Kleo::KeyFilter>)));
-    connect(this, SIGNAL(enableChangeStringFilter(bool)),
-            sb, SLOT(setChangeStringFilterEnabled(bool)));
-    connect(this, SIGNAL(enableChangeKeyFilter(bool)),
-            sb, SLOT(setChangeKeyFilterEnabled(bool)));
+    connect(sb, &SearchBar::keyFilterChanged, this, &TabWidget::setKeyFilter);
+    connect(this, &TabWidget::keyFilterChanged, sb, &SearchBar::setKeyFilter);
+
+    connect(this, &TabWidget::enableChangeStringFilter, sb, &SearchBar::setChangeStringFilterEnabled);
+    connect(this, &TabWidget::enableChangeKeyFilter, sb, &SearchBar::setChangeKeyFilterEnabled);
 }
 
 #include "moc_tabwidget.cpp"
